import {AfterViewInit, Component, EventEmitter, Output, ViewChild} from "@angular/core";
import {
  FileManagerComponent,
  NavigationPaneService,
  DetailsViewService
} from "@syncfusion/ej2-angular-filemanager";
import * as $ from "jquery";
import { Ajax } from "@syncfusion/ej2-base";
import {
  LinkService,
  ImageService,
  HtmlEditorService,
  RichTextEditorComponent
} from "@syncfusion/ej2-angular-richtexteditor";
import { DocumentEditorContainerComponent } from "@syncfusion/ej2-angular-documenteditor";
import {
  PdfViewerComponent,
  ToolbarService,
  LinkAnnotationService,
  BookmarkViewService,
  MagnificationService,
  ThumbnailViewService,
  NavigationService,
  TextSearchService,
  TextSelectionService,
  PrintService,
  AnnotationService,
  FormFieldsService
} from "@syncfusion/ej2-angular-pdfviewer";
import {
  SpreadsheetComponent,
  SelectEventArgs
} from "@syncfusion/ej2-angular-spreadsheet";

@Component({
  selector: "app-root",
  templateUrl: "./app.component.html",
  providers: [
    NavigationPaneService,
    DetailsViewService,
    ToolbarService,
    LinkService,
    ImageService,
    HtmlEditorService,
    LinkAnnotationService,
    BookmarkViewService,
    MagnificationService,
    ThumbnailViewService,
    ToolbarService,
    NavigationService,
    TextSearchService,
    TextSelectionService,
    PrintService,
    AnnotationService,
    FormFieldsService
  ]
})

export class AppComponent  {
  @ViewChild("rteInstance", { static: false })
  public rteObj: RichTextEditorComponent;
  @ViewChild("documenteditor_default", { static: false })
  public container: DocumentEditorContainerComponent;
  @ViewChild("pdfView", { static: false })
  public pdfView: PdfViewerComponent;
  @ViewChild("spreadsheet", { static: false })
  public spreadsheetObj: SpreadsheetComponent;

  //File Manager proeprties initialized
  public ajaxSettings: object;
  public view: string;
  // Services URL (Replace the URL with hosted URL of the sample Service)
  public hostUrl: string = "http://localhost:62869/";
  //PDFViewer Service URL
  public service: string = this.hostUrl + "api/pdfviewer";

  //SpreatSheet URL
  public openUrl: string = this.hostUrl + "api/FileManager/OpenExcel";
  public saveUrl: string = this.hostUrl + "api/FileManager/SaveExcel";
  public items = [
    "Undo",
    "Redo",
    "Separator",
    "Image",
    "Table",
    "Hyperlink",
    "Bookmark",
    "Comments",
    "TableOfContents",
    "Separator",
    "Header",
    "Footer",
    "PageSetup",
    "PageNumber",
    "Break",
    "Separator",
    "Find",
    "Separator",
    "LocalClipboard"
  ];
  public ngOnInit(): void {
    this.ajaxSettings = {
      url: this.hostUrl + "api/FileManager/FileOperations",
      getImageUrl: this.hostUrl + "api/FileManager/GetImage",
      uploadUrl: this.hostUrl + "api/FileManager/Upload",
      downloadUrl: this.hostUrl + "api/FileManager/Download"
    };
    this.view = "Details";
  }

  // File Manager File Open event
  fileOpen(args) {
    let fileName: string = args.fileDetails.name;
    let filePath: string =
      args.fileDetails.filterPath.replace(/\\/g, "/") + fileName;
    let fileType: string = args.fileDetails.type;
    // Document file
    if (
      fileType == ".docx" ||
      fileType == ".txt" ||
      fileType == ".html" ||
      fileType == ".rtf" ||
      fileType == ".xml"
    ) {
      this.showDocViewer(fileName);
      this.getFileStream(filePath, false);
    }
    // File Type is pdf
    else if (fileType == ".pdf") {
      this.showPDFViewer(fileName);
      // Open the file in PDF viewer
      this.getFileStream(filePath, true);
    }
    // File type is excel
    else if (fileType == ".xlsx") {
      this.showExcelViewer(fileName);
      // Open the file in spreadsheet.
      this.getBlob(fileName, filePath);
    }
  }
  created(args) {
    this.container.documentEditor.isReadOnly = true;
  }
  //Shows the Document viewer
  showDocViewer(name: string) {
    document.getElementsByClassName("doc-title")[0].innerHTML = name;
    (<HTMLElement>(
      document.getElementsByClassName("file-container")[0]
    )).style.visibility = "hidden";
    (<HTMLElement>(
      document.getElementsByClassName("doc-container")[0]
    )).style.visibility = "visible";
  }
  //Shows the PDF viewer
  showPDFViewer(name: string) {
    (<HTMLElement>(
      document.getElementsByClassName("file-container")[0]
    )).style.visibility = "hidden";
    (<HTMLElement>(
      document.getElementsByClassName("pdf-container")[0]
    )).style.visibility = "visible";
    document.getElementsByClassName("pdf-title")[0].innerHTML = name;
  }
  //Shows the SpreadSheet viewer
  showExcelViewer(name: string) {
    (<HTMLElement>(
      document.getElementsByClassName("file-container")[0]
    )).style.visibility = "hidden";
    (<HTMLElement>(
      document.getElementsByClassName("excel-container")[0]
    )).style.visibility = "visible";
    document.getElementsByClassName("excel-title")[0].innerHTML = name;
  }
  // sends HTTPPost Request to read the file as fileStream.
  getFileStream(filePath: string, isPDF: boolean) {
    let ajax: XMLHttpRequest = new XMLHttpRequest();
    ajax.open("POST", this.hostUrl + "api/FileManager/GetDocument", true);
    ajax.setRequestHeader("content-type", "application/json");
    ajax.onreadystatechange = () => {
      if (ajax.readyState === 4) {
        if (ajax.status === 200 || ajax.status === 304) {
          if (!isPDF) {
            // open SFDT text in document editor
            this.container.documentEditor.open(ajax.responseText);
          } else {
            //opens the file in pdf viewer
            this.pdfView.load(ajax.responseText, null);
          }
        }
      }
    };
    ajax.send(
      JSON.stringify({
        FileName: filePath,
        Action: !isPDF ? "ImportFile" : "LoadPDF"
      })
    );
  }
  //Get the blob using HTTPGet and loads in spreadsheet
  getBlob(fileName: string, filePath: string) {
    let request: XMLHttpRequest = new XMLHttpRequest();
    request.responseType = "blob";
    request.onload = () => {
      let file: any = new File([request.response], fileName);
      this.spreadsheetObj.open({ file: file });
    };
    request.open(
      "GET",
      this.hostUrl + "api/FileManager/GetExcel" + "?FileName=" + filePath
    );
    request.send();
  }

  //closes the viewers and open file manager
  onClicked() {
    (<HTMLElement>(
      document.getElementsByClassName("file-container")[0]
    )).style.visibility = "visible";
    (<HTMLElement>(
      document.getElementsByClassName("doc-container")[0]
    )).style.visibility = "hidden";
    (<HTMLElement>(
      document.getElementsByClassName("pdf-container")[0]
    )).style.visibility = "hidden";
    (<HTMLElement>(
      document.getElementsByClassName("excel-container")[0]
    )).style.visibility = "hidden";
  }

  //SpreadSheet Functions
  public customArgs: any;

  onOpenComplete(args: any) {
    this.customArgs = args.response.data.customArgs;
  }

  onSelect(args: SelectEventArgs) {
    for (var i = 0; i < this.customArgs.length; i++) {
      if (
        (this.spreadsheetObj as any).activeSheetTab - 1 ===
          this.customArgs[i].sheetIndex &&
        args.range ===
          this.customArgs[i].address + ":" + this.customArgs[i].address
      ) {
        document.getElementById("customvalue").innerHTML = this.customArgs[
          i
        ].customValue;
        break;
      }
    }
  }
}
